@@ -10,7 +10,7 @@ from .status import Status
 from . import test_case
 from .thread import Thread
 
-__version__ = '1.0.0'
+__version__ = '1.1.0'
 
 
 def judge(processor, source, tests, config=None,
@@ -24,9 +24,8 @@ def judge(processor, source, tests, config=None,
         container.remove(force=True)
 
 
-def run(container, processor, source, tests, config=None):
-    'Compile and judge'
-    config = config or {}
+def compile_source_code(container, processor, source, config):
+    'Compile the source file'
     container.exec_run(f"mkdir -p {processor.workdir}/0")
     put_bin(
         container,
@@ -40,17 +39,18 @@ def run(container, processor, source, tests, config=None):
     if 'compile' in config.get('callback', {}):
         config['callback']['compile'](exec_result.exit_code,
                                       exec_result.output)
-    if exec_result.exit_code:
-        return [[[Status.CE, exec_result.output, .0]] * len(tests),
-                exec_result.output]
     exec_run(container, processor.after_compile, f'{processor.workdir}/0')
+    return exec_result
 
+
+def judge_test_cases(container, processor, tests, config):
+    'Judge test cases'
+    threads_limit = config.get('threads') or len(tests)
     res = []
-    for i in range(ceil(len(tests) / (config.get('threads') or len(tests)))):
+    for i in range(ceil(len(tests) / threads_limit)):
         threads = []
-        for j in range(i * (config.get('threads') or len(tests)),
-                       min((i + 1) * (config.get('threads') or len(tests)),
-                           len(tests))):
+        for j in range(i * threads_limit,
+                       min((i + 1) * threads_limit, len(tests))):
             threads.append(
                 Thread(
                     target=test_case.__init__,
@@ -62,4 +62,16 @@ def run(container, processor, source, tests, config=None):
         for thread in threads:
             thread.join()
             res.append(thread.return_value)
+    return res
+
+
+def run(container, processor, source, tests, config=None):
+    'Compile and judge'
+    config = config or {}
+    exec_result = compile_source_code(container, processor, source, config)
+    if exec_result.exit_code:
+        return [[[Status.CE, (None, None), .0]] * len(tests),
+                exec_result.output]
+
+    res = judge_test_cases(container, processor, tests, config)
     return [res, exec_result.output]
